@@ -1,5 +1,5 @@
 use std::io::Read;
-use std::path::{Path, PathBuf};
+use std::path::PathBuf;
 
 use anyhow::{anyhow, Result};
 use bytes::Bytes;
@@ -17,6 +17,7 @@ pub mod local;
 pub mod node;
 pub mod rclone;
 pub mod rest;
+pub mod stdin;
 
 pub use self::ignore::*;
 pub use cache::*;
@@ -28,6 +29,7 @@ pub use local::*;
 use node::Node;
 pub use rclone::*;
 pub use rest::*;
+pub use stdin::*;
 
 /// All [`FileType`]s which are located in separated directories
 pub const ALL_FILE_TYPES: [FileType; 4] = [
@@ -144,10 +146,24 @@ pub trait WriteBackend: ReadBackend {
     fn remove(&self, tpe: FileType, id: &Id, cacheable: bool) -> Result<()>;
 }
 
-pub trait ReadSource: Iterator<Item = Result<(PathBuf, Node)>> {
-    type Reader: Read;
-    fn read(path: &Path) -> Result<Self::Reader>;
-    fn size(&self) -> Result<u64>;
+pub struct ReadSourceEntry<O> {
+    pub path: PathBuf,
+    pub node: Node,
+    pub open: Option<O>,
+}
+
+pub trait ReadSourceOpen {
+    type Reader: Read + Send + 'static;
+
+    fn open(self) -> Result<Self::Reader>;
+}
+
+pub trait ReadSource {
+    type Open: ReadSourceOpen;
+    type Iter: Iterator<Item = Result<ReadSourceEntry<Self::Open>>>;
+
+    fn size(&self) -> Result<Option<u64>>;
+    fn entries(self) -> Self::Iter;
 }
 
 pub trait WriteSource: Clone {
