@@ -1,21 +1,65 @@
 use std::fmt;
+use std::path::PathBuf;
 use std::str::FromStr;
 use std::{cmp::Ordering, fmt::Display};
 
 use anyhow::{anyhow, bail, Result};
-use chrono::{DateTime, Local};
-use clap::Parser;
+use chrono::{DateTime, Duration, Local};
+use clap::{AppSettings, Parser};
 use derivative::Derivative;
+use dunce::canonicalize;
+use gethostname::gethostname;
 use indicatif::ProgressBar;
 use itertools::Itertools;
 use log::*;
 use merge::Merge;
+use path_dedot::ParseDot;
+use rhai::serde::to_dynamic;
+use rhai::{Dynamic, Engine, FnPtr, AST};
 use serde::{Deserialize, Serialize};
 use serde_with::{serde_as, DisplayFromStr};
 
 use super::Id;
 use crate::backend::{DecryptReadBackend, FileType, RepoFile};
 
+#[serde_as]
+#[derive(Clone, Default, Parser, Deserialize, Merge)]
+#[clap(global_setting(AppSettings::DeriveDisplayOrder))]
+#[serde(default, rename_all = "kebab-case", deny_unknown_fields)]
+pub struct SnapshotOptions {
+    /// Label snapshot with given label
+    #[clap(long, value_name = "LABEL")]
+    label: Option<String>,
+
+    /// Tags to add to snapshot (can be specified multiple times)
+    #[clap(long, value_name = "TAG[,TAG,..]")]
+    #[serde_as(as = "Vec<DisplayFromStr>")]
+    #[merge(strategy = merge::vec::overwrite_empty)]
+    tag: Vec<StringList>,
+
+    /// Add description to snapshot
+    #[clap(long, value_name = "DESCRIPTION")]
+    description: Option<String>,
+
+    /// Add description to snapshot from file
+    #[clap(long, value_name = "FILE", conflicts_with = "description")]
+    description_from: Option<PathBuf>,
+
+    /// Mark snapshot as uneraseable
+    #[clap(long, conflicts_with = "delete-after")]
+    #[merge(strategy = merge::bool::overwrite_false)]
+    delete_never: bool,
+
+    /// Mark snapshot to be deleted after given duration (e.g. 10d)
+    #[clap(long, value_name = "DURATION")]
+    #[serde_as(as = "Option<DisplayFromStr>")]
+    delete_after: Option<humantime::Duration>,
+
+    /// Set the host name manually
+    #[clap(long, value_name = "NAME")]
+    host: Option<String>,
+}
+
 /// This is an extended version of the summaryOutput structure of restic in
 /// restic/internal/ui/backup$/json.go
 #[derive(Debug, Clone, Serialize, Deserialize, Derivative)]
@@ -49,6 +93,16 @@ pub struct SnapshotSummary {
     pub backup_duration: f64, // in seconds
 }
 
+impl SnapshotSummary {
+    pub fn finalize(&mut self, snap_time: DateTime<Local>) -> Result<()> {
+        let end_time = Local::now();
+        self.backup_duration = (end_time - self.backup_start).to_std()?.as_secs_f64();
+        self.total_duration = (end_time - snap_time).to_std()?.as_secs_f64();
+        self.backup_end = end_time;
+        Ok(())
+    }
+}
+
 #[derive(Debug, Clone, PartialEq, Eq, Serialize, Deserialize, Derivative)]
 #[derivative(Default)]
 pub enum DeleteOption {
@@ -106,6 +160,51 @@ impl RepoFile for SnapshotFile {
 }
 
 impl SnapshotFile {
+    pub fn new_from_options(
+        opts: SnapshotOptions,
+        time: DateTime<Local>,
+        command: String,
+    ) -> Result<Self> {
+        let hostname = match opts.host {
+            Some(host) => host,
+            None => {
+                let hostname = gethostname();
+                hostname
+                    .to_str()
+                    .ok_or_else(|| anyhow!("non-unicode hostname {:?}", hostname))?
+                    .to_string()
+            }
+        };
+
+        let delete = match (opts.delete_never, opts.delete_after) {
+            (true, _) => DeleteOption::Never,
+            (_, Some(d)) => DeleteOption::After(time + Duration::from_std(*d)?),
+            (false, None) => DeleteOption::NotSet,
+        };
+
+        let mut snap = SnapshotFile {
+            time,
+            hostname,
+            label: opts.label.unwrap_or_default(),
+            delete,
+            summary: Some(SnapshotSummary {
+                command,
+                ..Default::default()
+            }),
+            description: opts.description,
+            ..Default::default()
+        };
+
+        // use description from description file if it is given
+        if let Some(file) = opts.description_from {
+            snap.description = Some(std::fs::read_to_string(file)?);
+        }
+
+        snap.set_tags(opts.tag.clone());
+
+        Ok(snap)
+    }
+
     fn set_id(tuple: (Id, Self)) -> Self {
         let (id, mut snap) = tuple;
         snap.id = id;
@@ -242,6 +341,19 @@ impl SnapshotFile {
     }
 
     pub fn matches(&self, filter: &SnapshotFilter) -> bool {
+        if let Some(filter_fn) = &filter.filter_fn {
+            match filter_fn.call::<bool>(self) {
+                Ok(result) => {
+                    if !result {
+                        return false;
+                    }
+                }
+                Err(err) => {
+                    warn!("Error evaluating filter-fn for snapshot {}: {err}", self.id);
+                }
+            }
+        }
+
         self.paths.matches(&filter.filter_paths)
             && self.tags.matches(&filter.filter_tags)
             && (filter.filter_host.is_empty() || filter.filter_host.contains(&self.hostname))
@@ -307,9 +419,28 @@ impl Ord for SnapshotFile {
     }
 }
 
+struct SnapshotFn(FnPtr, AST);
+impl FromStr for SnapshotFn {
+    type Err = anyhow::Error;
+    fn from_str(s: &str) -> Result<Self> {
+        let engine = Engine::new();
+        let ast = engine.compile(s)?;
+        let func = engine.eval_ast::<FnPtr>(&ast)?;
+        Ok(Self(func, ast))
+    }
+}
+
+impl SnapshotFn {
+    fn call<T: Clone + Send + Sync + 'static>(&self, sn: &SnapshotFile) -> Result<T> {
+        let engine = Engine::new();
+        let sn: Dynamic = to_dynamic(sn)?;
+        Ok(self.0.call::<T>(&engine, &self.1, (sn,))?)
+    }
+}
+
 #[serde_as]
 #[derive(Default, Parser, Deserialize, Merge)]
-#[serde(default, rename_all = "kebab-case")]
+#[serde(default, rename_all = "kebab-case", deny_unknown_fields)]
 pub struct SnapshotFilter {
     /// Hostname to filter (can be specified multiple times)
     #[clap(long, value_name = "HOSTNAME")]
@@ -332,6 +463,11 @@ pub struct SnapshotFilter {
     #[serde_as(as = "Vec<DisplayFromStr>")]
     #[merge(strategy=merge::vec::overwrite_empty)]
     filter_tags: Vec<StringList>,
+
+    /// Function to filter snapshots
+    #[clap(long, value_name = "FUNC")]
+    #[serde_as(as = "Option<DisplayFromStr>")]
+    filter_fn: Option<SnapshotFn>,
 }
 
 #[derive(Clone, Default, Deserialize)]
@@ -454,6 +590,18 @@ impl StringList {
         }
     }
 
+    pub fn set_paths(&mut self, paths: &[PathBuf]) -> Result<()> {
+        self.0 = paths
+            .iter()
+            .map(|p| {
+                Ok(p.to_str()
+                    .ok_or_else(|| anyhow!("non-unicode path {:?}", p))?
+                    .to_string())
+            })
+            .collect::<Result<Vec<_>>>()?;
+        Ok(())
+    }
+
     pub fn remove_all(&mut self, string_lists: Vec<StringList>) {
         self.0
             .retain(|s| !string_lists.iter().any(|sl| sl.contains(s)));
@@ -466,4 +614,86 @@ impl StringList {
     pub fn formatln(&self) -> String {
         self.0.join("\n")
     }
+
+    pub fn iter(&self) -> std::slice::Iter<String> {
+        self.0.iter()
+    }
+}
+
+#[derive(Default, Debug, PartialEq, Eq, PartialOrd, Ord, Clone)]
+pub struct PathList(Vec<PathBuf>);
+
+impl Display for PathList {
+    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
+        if !self.0.is_empty() {
+            write!(f, "{:?}", self.0[0])?;
+        }
+        for p in &self.0[1..] {
+            write!(f, ",{:?}", p)?;
+        }
+        Ok(())
+    }
+}
+
+impl PathList {
+    pub fn from_strings<I>(source: I, sanitize: bool) -> Result<Self>
+    where
+        I: IntoIterator,
+        I::Item: AsRef<str>,
+    {
+        let mut paths = PathList(
+            source
+                .into_iter()
+                .map(|source| PathBuf::from(source.as_ref()))
+                .collect(),
+        );
+
+        if sanitize {
+            paths.sanitize()?;
+        }
+        paths.merge_paths();
+        Ok(paths)
+    }
+
+    pub fn len(&self) -> usize {
+        self.0.len()
+    }
+
+    pub fn from_string(sources: &str, sanitize: bool) -> Result<Self> {
+        Self::from_strings(sources.split_whitespace(), sanitize)
+    }
+
+    pub fn paths(&self) -> Vec<PathBuf> {
+        self.0.clone()
+    }
+
+    // sanitize paths: parse dots and absolutize if needed
+    fn sanitize(&mut self) -> Result<()> {
+        for path in &mut self.0 {
+            *path = path.parse_dot()?.to_path_buf();
+        }
+        if self.0.iter().any(|p| p.is_absolute()) {
+            for path in &mut self.0 {
+                *path = canonicalize(&path)?;
+            }
+        }
+        Ok(())
+    }
+
+    // sort paths and filters out subpaths of already existing paths
+    fn merge_paths(&mut self) {
+        // sort paths
+        self.0.sort_unstable();
+
+        let mut root_path = None;
+
+        // filter out subpaths
+        self.0.retain(|path| match &root_path {
+            Some(root_path) if path.starts_with(root_path) => false,
+            _ => {
+                root_path = Some(path.to_path_buf());
+                true
+            }
+        });
+    }
 }
